function [t_aporx, K, K_inside_R] = calc_scatter_trafo(DN, DN_background, trunc_rad, J, elec_position_angle,grid_size)
%Calculates the exponential approximation of the scattering transform:

%DN <number of electrodes - 1>*<number of electrodes - 1>:
% Dirichlet-to-Neumann-Map

%DN_background <number of electrodes - 1>*<number of electrodes - 1>:
% Dirichlet-to-Neumann-Map

%trunc_rad <1>: truncation radius

%J <number of electrodes>*<number of linear independend current patterns>:
% stimulation current pattern

%elec_position_angle <number of electrodes>:
% angle of the electrode measured from the center and the x-axis

%grid_size <1>:
% sets the size of K (<size K> = 2^grid_size)

%compute the grid size
M = 2^grid_size;
%step size between each point in K
stepsize_grid = 2*trunc_rad/(M - 1);
grid_base_vec = [-trunc_rad:stepsize_grid:trunc_rad];
[K_x, K_y] = meshgrid(grid_base_vec);
%creation of K
K = K_x + 1i*K_y;
%boolean mask of which points are actually inside the truncation radius
K_inside_R = K( abs(K) < trunc_rad);

del_L = (DN -DN_background);

%computation of the scattering transform 
t_aporx = zeros(length(K_inside_R),1);
for i = 1:length(K_inside_R)
    k = K_inside_R(i);
    c_k = (full(J))'*exp(1i*k*exp(1i*elec_position_angle));%%electrode_pos
    d_k = (full(J))'*exp(1i*conj(k)*conj(exp(1i*elec_position_angle)));
    
    t_aporx(i) = conj(d_k)'*(del_L*c_k);
end

end