function [mu_final] = solve_d_bar(K, trunc_rad, t_aporx, recon_cords)
%Solves the D-Bar equation using:
% Note the loop for solving all points can be made parallel if the parallel
% computing toolbox is available

%K <2^grid_size>*<2^grid_size>: the K-Grid computed by calc_scatter_trafo

%trunc_rad <1>: truncation radius R

%t_aprox <number of points inside R>: approximate scattering transform for
%values inside R

%recon_cords <number of points to recosntruct>*<2>: points to reconstruct
%with x, y per point

%calculation of the slightly larger Grid Q (comapred to K) for the Greens function
M = size(K,1) ;
begin_Q = trunc_rad*((2*M - 1)/(M - 1));
grid_vec_Q = [-begin_Q:2*trunc_rad/(M-1):begin_Q];
[Q_x, Q_y] = meshgrid(grid_vec_Q);
Q = Q_x + 1i*Q_y;

% factor that decides how big the smoothing zone outside of R is
smooth_zone_factor = 1/4;
% calcualte G bar
G_bar = 1./(pi*Q);
% set everything to 0 outisde the smoothing zone
G_bar(abs(Q) > trunc_rad*(2 + smooth_zone_factor)) = 0;
% get index of the smoothing zone
smooth_zone = abs(Q) > trunc_rad*(2) & abs(Q) < trunc_rad*(2 + smooth_zone_factor);
% apply the smoothing (in this case linear) to G_bar
G_bar(smooth_zone) = G_bar(smooth_zone).*((trunc_rad*(2 + smooth_zone_factor) - abs(Q(smooth_zone)))/(trunc_rad*smooth_zone_factor));

% make the scattering transform compatible to the larger grid Q
scat = zeros(size(K));
rad_idx = abs(K) < trunc_rad;
scat(rad_idx) = t_aporx;
scat = padarray(scat, [M/2 M/2], 0, 'both');

% use the FFT trick by Mueller and Siltanen
%(due to Matlabs indexing fftshift is needed before apllying fft2)
G_bar_fft = fft2(fftshift(G_bar));

% calcualte the first part of T (this is independnt of the location z)
T_R_first = scat./(4*pi*conj(Q));


%% Stuff needed for the computation, this is not directly related to the D-Bar algorithm
% index of which vlaues are inside R
rad_idx_dbar = abs(Q) < trunc_rad;
% number of points inside R
num_pts_in_R = sum(rad_idx_dbar(:));

% Construct right hand side of the Dbar equation as the initial guess
right_side_eq = [ones(num_pts_in_R,1);zeros(num_pts_in_R,1)];

% transform the reconstruction points into complex coordiantes
recon_pts = recon_cords(:, 1) + 1i*recon_cords(:, 2);
num_rec_pts  = length(recon_pts);

mu_final = ones(num_rec_pts,1);
iniguess = [ones(num_pts_in_R,1);zeros(num_pts_in_R,1)];


middle_val = size(T_R_first, 1)/2;
%% When you have the parralel computation toolkit feel free to use parfor instead of for
for iii = 1:num_rec_pts
    % get positions for recosntruction
    z = recon_pts(iii);
    
    % multiplicator for the D-Bar function
    T_R = T_R_first.*(exp(-1i*(Q*z+conj(Q*z))));
    
    % Solve D-Bar equation; Note that instead of a matrix a function is
    % given, the input aargument for that function follow after the
    % second []
    [sol, ~] = gmres('DB_oper_demyst', right_side_eq, 40, 1e-4, 450, [], [], M*2, rad_idx_dbar, iniguess, num_pts_in_R, 2*trunc_rad/(M - 1), G_bar_fft, T_R)
    
    %use sol to get mu for the recosntruction
    mu = zeros(size(Q));
    mu(Rind_true) = sol(1:num_pts_in_R) + 1i*sol((num_pts_in_R+1):end);
    
    % get mu from interpolation of the center
    mu_final(iii) = mean([mu(middle_val,middle_val), mu(middle_val+1,middle_val), mu(middle_val, middle_val+1), mu(middle_val+1, middle_val+1)]);
    
end

end